'use strict';

var getTTL = function getTTL(lifespan) {
  var currentTime = new Date().getTime();
  return lifespan > 0 ? currentTime + lifespan : currentTime;
};
var isValidTTL = function isValidTTL(timestamp) {
  var currentTime = new Date().getTime();
  return currentTime <= timestamp;
};
var isObject = function isObject(val) {
  return val === Object(val);
};
var getObjectFromString = function getObjectFromString(string) {
  if (typeof string === "number") {
    return null;
  }

  try {
    return JSON.parse(string);
  } catch (e) {
    return null;
  }
};

var loadFromStorage = function loadFromStorage(_ref) {
  var storageFn = _ref.storageFn,
      itemName = _ref.itemName;

  try {
    var storage = storageFn();
    var rawItem = storage.getItem(itemName);

    if (rawItem == null) {
      return null;
    }

    var wrapper = getObjectFromString(rawItem);

    if (wrapper == null) {
      return rawItem;
    }

    if (wrapper.ttl != null) {
      if (isValidTTL(wrapper.ttl)) {
        return wrapper.item;
      }

      storage.removeItem(itemName);
      return null;
    }

    return wrapper;
  } catch (err) {
    return null;
  }
};
var saveInStorage = function saveInStorage(_ref2) {
  var storageFn = _ref2.storageFn,
      item = _ref2.item,
      itemName = _ref2.itemName,
      lifespan = _ref2.lifespan;

  if (lifespan === 0) {
    return false;
  }

  try {
    var storage = storageFn();

    if (lifespan == null) {
      storage.setItem(itemName, isObject(item) ? JSON.stringify(item) : item);
    } else {
      var ttl = getTTL(lifespan);
      var wrapper = {
        ttl: ttl,
        item: item
      };
      storage.setItem(itemName, JSON.stringify(wrapper));
    }

    return true;
  } catch (err) {
    return false;
  }
};
var updateTtl = function updateTtl(_ref3) {
  var storageFn = _ref3.storageFn,
      itemName = _ref3.itemName,
      lifespan = _ref3.lifespan;

  try {
    var storage = storageFn();
    var rawItem = storage.getItem(itemName);

    if (rawItem && lifespan && lifespan !== 0) {
      var wrapper = getObjectFromString(rawItem);

      if (wrapper && wrapper.ttl != null) {
        if (isValidTTL(wrapper.ttl)) {
          saveInStorage({
            storageFn: storageFn,
            item: wrapper.item,
            itemName: itemName,
            lifespan: lifespan
          });
          return true;
        }

        return false;
      }
    }
  } catch (err) {// continue regardless of error
  }

  return false;
};
var hasItem = function hasItem(_ref4) {
  var storageFn = _ref4.storageFn,
      itemName = _ref4.itemName;
  return loadFromStorage({
    storageFn: storageFn,
    itemName: itemName
  }) != null;
};

/* eslint-disable no-undef */

var initializeForStorage = function initializeForStorage(storageFn) {
  return {
    load: function load(itemName) {
      return loadFromStorage({
        storageFn: storageFn,
        itemName: itemName
      });
    },
    set: function set(itemName, item, lifespan) {
      return saveInStorage({
        storageFn: storageFn,
        itemName: itemName,
        item: item,
        lifespan: lifespan
      });
    },
    setLifespan: function setLifespan(itemName, lifespan) {
      return updateTtl({
        storageFn: storageFn,
        itemName: itemName,
        lifespan: lifespan
      });
    },
    has: function has(itemName) {
      return hasItem({
        storageFn: storageFn,
        itemName: itemName
      });
    }
  };
};

var buildNewStorageWrapper = (function () {
  return {
    local: initializeForStorage(function () {
      return localStorage;
    }),
    session: initializeForStorage(function () {
      return sessionStorage;
    })
  };
});

var storage = buildNewStorageWrapper();

module.exports = storage;
